* Output options: all can be turned on/off by adding/ removing the NO;
  OPTIONS NOnumber NOdate NOcenter FormDlim=' ' PageSize=MAX LineSize=MAX NOfmterr;
* Remove default title, turn on new HTML graphics;
  TITLE; ODS GRAPHICS ON; ODS LISTING;  

********************************************************************************;
*******    FIRST DEFINE "FILESAVE" PLACEHOLDER FOR FOLDER LOCATION       *******;
********************************************************************************;

* Paste in the folder address where "GSS_Example.xlsx" is saved after = before ;
%LET filesave= C:\Dropbox\21SP_PSQF6242\PSQF6242_Example4;
* Using UIowa virtual desktop instead;
*%LET filesave= \\Client\C:\Dropbox\21SP_PSQF6242\PSQF6242_Example2;

* We can then refer to the filesave place by putting & in front and . behind it;

* IMPORT GSS_Example.xlsx data using filesave reference and exact file name;
* from the Excel workbook in DATAFILE= location from SHEET= ;
* New SAS file is in "work" library place with name "Example4";
* "GETNAMES" reads in the first row as variable names;
* DBMS=XLSX (can also use EXCEL or XLS for .xls files);
PROC IMPORT DATAFILE="&filesave.\GSS_Example.xlsx" 
            OUT=work.Example4 DBMS=XLSX REPLACE; 
     SHEET="GSS_Example"; 
     GETNAMES=YES; 
RUN;

* Create formats: set of value labels for categorical variables;
PROC FORMAT; 
     VALUE Fclass 1="1.Lower" 2="2.Middle" 3="3.Upper";
     VALUE Fhappy 1="1.Unhappy" 2="2.Neither" 3="3.Fairly Happy" 
	                4="4.Very Happy" 5="5.Completely Happy";
RUN;

* All data transformations must happen inside a DATA+SET combo to know where to use them;
* Here is how to make a new variable: new = old;
DATA work.Example4; SET work.Example4;
* Label variables and apply value formats for variables used below;
* LABEL name=      "name: Descriptive Variable Label";
  LABEL workclass= "workclass: 3-Category Working Class"
        age=       "age: Years of Age"
        educ=      "educ: Years of Education"
        happy=     "happy: 5-Category Happy Rating"
        income=    "income: Annual Income in 1000s";
* Apply value labels created above: name Format.;
  FORMAT workclass Fclass. happy Fhappy.; 
* Select cases complete on all variables to be used;
  WHERE NMISS(income,workclass,age,educ,happy)=0;
RUN;
* Now dataset work.Example4 is ready to use; 

* Open output directory to save results to;
ODS RTF FILE="&filesave.\Example4_SAS_Output.rtf" 
        STYLE=HTMLBlue STARTPAGE=NO BODYTITLE;


******************************************************************************;
*******       BEGIN DATA DESCRIPTION AND EMPTY MODEL FOR INCOME        *******;
******************************************************************************;

TITLE "SAS Descriptive Statistics for Quantitative income";
PROC MEANS NDEC=3 NOLABELS N MEAN STDDEV VAR MIN MAX DATA=work.Example4; 
     VAR income;
RUN; TITLE;

* Histograms to visualize quantitative variables;
* NOPRINT spares the rest of the results I do not want right now;
TITLE "SAS Histogram of Quantitative income";
PROC UNIVARIATE NOPRINT DATA=work.Example4;
     VAR income; 
     HISTOGRAM income / MIDPOINTS=0 TO 70 BY 5; 
RUN; QUIT; TITLE;

TITLE "SAS GLM Empty Model Predicting Income";
PROC GLM DATA=work.Example4 NAMELEN=100; 
     MODEL income = / SOLUTION ALPHA=.05 CLPARM;
RUN; QUIT; TITLE;


******************************************************************************;
*******       BEGIN PREDICTION OF INCOME BY 3-CATEGORY WORKCLASS       *******;
******************************************************************************;

TITLE "SAS Descriptive Statistics for Categorical workclass";
PROC FREQ DATA=work.Example4; 
     TABLE workclass; 
RUN; TITLE;

* SAS code to create dummy-coded binary predictors;
DATA work.Example4; SET work.Example4;
     LvsM=.; LvsU=.; * Make new empty variables;
     IF workclass=1 THEN DO; LvsM=0; LvsU=0; END; * Replace each for lower;
     IF workclass=2 THEN DO; LvsM=1; LvsU=0; END; * Replace each for middle;
     IF workclass=3 THEN DO; LvsM=0; LvsU=1; END; * Replace each for upper; 
     LABEL LvsM="LvsM: Low=0 vs Mid=1 Class"
           LvsU="LvsU: Low=0 vs Upp=1 Class";
RUN;

TITLE "SAS GLM Predicting Income from 3-Category workclass";
PROC GLM DATA=work.Example4 NAMELEN=100; * PLOTS(UNPACK)=DIAGNOSTICS;
     MODEL income = LvsM LvsU / SOLUTION ALPHA=.05 CLPARM;
	* Ask for predicted income per group and group differences;
     ESTIMATE "Low Mean"    intercept 1 LvsM  0 LvsU 0;
     ESTIMATE "Mid Mean"    intercept 1 LvsM  1 LvsU 0;
     ESTIMATE "Upp Mean"    intercept 1 LvsM  0 LvsU 1;
     ESTIMATE "Low vs Mid Diff"         LvsM  1 LvsU 0;
     ESTIMATE "Low vs Upp Diff"         LvsM  0 LvsU 1;
     ESTIMATE "Mid vs Upp Diff"         LvsM -1 LvsU 1;
     * Save requested estimates as SAS dataset to do math on them;
     ODS OUTPUT Estimates=work.ClassEstimates;
RUN; QUIT; TITLE;

* SAS code to compute Cohen d effect sizes for mean differences;
DATA work.ClassEstimates; SET work.ClassEstimates;
     CohenD=(2*tValue)/SQRT(731); * Number = DF denominator;
RUN;
* Print effect sizes;
TITLE "Cohen D Effect Sizes for 3-Category workclass";
PROC PRINT NOOBS DATA=work.ClassEstimates; 
     VAR Parameter--CohenD; 
RUN; TITLE;

TITLE "SAS PROC REG to get standardized slopes for workclass";
PROC REG DATA=work.Example4; 
     MODEL income = LvsM LvsU / STB;
RUN; QUIT; TITLE;


******************************************************************************;
*******       BEGIN PREDICTION OF INCOME BY QUADRATIC YEARS OF AGE     *******;
******************************************************************************;

TITLE "SAS Descriptive Statistics for Quantitative age";
PROC MEANS NDEC=3 NOLABELS N MEAN STDDEV VAR MIN MAX DATA=work.Example4; 
     VAR age;
RUN; TITLE;

* Histograms to visualize quantitative variables;
* NOPRINT spares the rest of the results I do not want right now;
TITLE "SAS Histogram of Quantitative age";
PROC UNIVARIATE NOPRINT DATA=work.Example4;
     VAR age;  
     HISTOGRAM age / MIDPOINTS=18 TO 86 BY 2; 
RUN; QUIT; TITLE;

* SAS code to make new age variable centered at 18 (minimum in sample);
DATA work.Example4; SET work.Example4;
     age18=age-18; 
     LABEL age18= "age18: Age (0=18 years)";
RUN;

TITLE "SAS GLM Predicting Income from Linear Centered Age (0=18)";
PROC GLM DATA=work.Example4 NAMELEN=100 PLOTS(UNPACK)=DIAGNOSTICS;
     MODEL income = age18 / SOLUTION ALPHA=.05 CLPARM;
     * Ask for predicted income for example ages;
     ESTIMATE "Pred Income Age 30 (age18=12)" intercept 1 age18 12;
     ESTIMATE "Pred Income Age 50 (sage18=32)" intercept 1 age18 32;
     ESTIMATE "Pred Income Age 70 (age18=52)" intercept 1 age18 52;
RUN; QUIT; TITLE; 

TITLE "SAS GLM Predicting Income from Quadratic Centered Age";
PROC GLM DATA=work.Example4 NAMELEN=100 PLOTS(UNPACK)=DIAGNOSTICS;
     * Asterisk creates multiplied predictor variable;
     MODEL income = age18 age18*age18 / SOLUTION ALPHA=.05 CLPARM;
     * Save predicted income and SE to new dataset to make pictures;
     OUTPUT OUT=work.PredIncomebyAge PREDICTED=YhatAge STDP=SEyhatAge;
     * Ask for predicted income for example ages;
     ESTIMATE "Pred Income Age 30 (age18=12)" intercept 1 age18 12 age18*age18 144;
     ESTIMATE "Pred Income Age 50 (age18=32)" intercept 1 age18 32 age18*age18 1024;
     ESTIMATE "Pred Income Age 70 (age18=52)" intercept 1 age18 52 age18*age18 2704;
     * Linear age slope changes by 2*quadratic coefficient, so multiply age*2;
     ESTIMATE "Pred Linear Age Slope Age 30 (age18=12)"   age18 1  age18*age18 24;
     ESTIMATE "Pred Linear Age Slope Age 50 (age18=32)"   age18 1  age18*age18 64;
     ESTIMATE "Pred Linear Age Slope Age 70 (age18=52)"   age18 1  age18*age18 104;
RUN; QUIT; TITLE; 

* Not doing standardized slopes because of ambiguity of interpreting 
  them in the presence of an interaction;

******************************************************************************;
*******   BEGIN PREDICTION OF INCOME BY PIECEWISE YEARS OF EDUCATION   *******;
******************************************************************************;

TITLE "SAS Descriptive Statistics for Quantitative Variable education";
PROC MEANS NDEC=3 NOLABELS N MEAN STDDEV VAR MIN MAX DATA=work.Example4; 
     VAR educ;
RUN; TITLE;

* Histograms to visualize quantitative variables;
* NOPRINT spares the rest of the results I do not want right now;
TITLE "SAS Histograms of Quantitative Variable education";
PROC UNIVARIATE NOPRINT DATA=work.Example4;
     VAR educ;  
     HISTOGRAM educ / MIDPOINTS=2 TO 20 BY 1;  
RUN; QUIT; TITLE;

* SAS code to make 3 new variables for sections of education;
DATA work.Example4; SET work.Example4;
     lessHS=.; gradHS=.; overHS=.; * Make three new empty variables;
     * Replace for educ less than 12;
     IF educ LT 12 THEN DO; lessHS=educ-11; gradHS=0; overHS=0;       END;
     * Replace for educ greater or equal to 12;
     IF educ GE 12 THEN DO; lessHS=0;       gradHS=1; overHS=educ-12; END;
     LABEL lessHS= "lessHS: Slope for Years Ed Less Than High School"
           gradHS= "gradHS: Bump for Graduating High School"
           overHS= "overHS: Slope for Years Ed After High School";
RUN;

TITLE "SAS GLM Predicting Income from Piecewise Education";
PROC GLM DATA=work.Example4 NAMELEN=100 PLOTS(UNPACK)=DIAGNOSTICS;
     MODEL income = lessHS gradHS overHS / SOLUTION ALPHA=.05 CLPARM;
     * Example of how to compare slopes;
     ESTIMATE "Diff in ed slope:  2-11 vs 11-12" lessHS -1 gradHS 1;
     ESTIMATE "Diff in ed slope: 11-12 vs 12-20" gradHS -1 overHS 1;
     * Save predicted income and SE to new dataset to make pictures;
     OUTPUT OUT=work.PredIncomebyEduc PREDICTED=YhatEduc STDP=SEyhatEduc;
RUN; QUIT; TITLE;

TITLE "SAS PROC REG to get standardized slopes for education";
PROC REG DATA=work.Example4;
     MODEL income = lessHS gradHS overHS / STB;
RUN; QUIT; TITLE;


******************************************************************************;
*******   BEGIN PREDICTION OF INCOME BY ORDINAL 5-CATEGORY HAPPINESS   *******;
******************************************************************************;

TITLE "SAS Descriptive Statistics for Categorical happy";
PROC FREQ DATA=work.Example4; 
     TABLE happy; 
RUN; TITLE;

* SAS code to make a single centered predictor for happy;
DATA work.Example4; SET work.Example4;
     happy1=happy-1;
     LABEL happy1= "happy1: Happy Catgeory (0=1)";
RUN;

TITLE "SAS GLM Predicting Income from Linear Centered Happy (0=1)";
PROC GLM DATA=work.Example4 NAMELEN=100 PLOTS(UNPACK)=DIAGNOSTICS;
     MODEL income = happy1 / SOLUTION ALPHA=.05 CLPARM;
	* Save predicted income and SE to new dataset to make pictures;
     OUTPUT OUT=work.PredIncomebyHappy1 PREDICTED=Yhat1Happy STDP=SEyhat1Happy;
RUN; QUIT; TITLE;

* SAS code to make 4 new varsiables for adjacent values of happy;
DATA work.Example4; SET work.Example4;
     h1v2=.; h2v3=.; h3v4=.; h4v5=.; * Make 4 new empty variables;
     IF happy=1 THEN DO; h1v2=0; h2v3=0; h3v4=0; h4v5=0; END;
     IF happy=2 THEN DO; h1v2=1; h2v3=0; h3v4=0; h4v5=0; END;
     IF happy=3 THEN DO; h1v2=1; h2v3=1; h3v4=0; h4v5=0; END;
     IF happy=4 THEN DO; h1v2=1; h2v3=1; h3v4=1; h4v5=0; END;
     IF happy=5 THEN DO; h1v2=1; h2v3=1; h3v4=1; h4v5=1; END;
     LABEL h1v2="Slope from Happy 1 to 2"
           h2v3="Slope from Happy 2 to 3"
           h3v4="Slope from Happy 3 to 4"
           h4v5="Slope from Happy 4 to 5";
RUN;

TITLE "SAS GLM Predicting Income from Piecewise Adjacent Slopes for Happy";
PROC GLM DATA=work.Example4 NAMELEN=100; * PLOTS(UNPACK)=DIAGNOSTICS;
     MODEL income = h1v2 h2v3 h3v4 h4v5 / SOLUTION ALPHA=.05 CLPARM;
     * Example of how to compare slopes;
     ESTIMATE "Diff in Slope 1-2 vs 2-3" h1v2 -1 h2v3 1;
     ESTIMATE "Diff in Slope 2-3 vs 3-4" h2v3 -1 h3v4 1;
     ESTIMATE "Diff in Slope 3-4 vs 4-5" h3v4 -1 h4v5 1;
RUN; QUIT; TITLE;

TITLE "SAS PROC REG to get standardized slopes for happy";
PROC REG DATA=work.Example4;
     MODEL income = h1v2 h2v3 h3v4 h4v5 / STB;
RUN; QUIT; TITLE;


******************************************************************************;
*******   CODE TO MAKE PICTURES FOR PREDICTED INCOME BY PREDICTOR      *******;
******************************************************************************;

* Save descriptive statistics for personal income by unqiue predictor value;
PROC MEANS NOPRINT NDEC=3 DATA=work.Example4; 
     CLASS age educ happy; WAYS 1; * WAYS=1 means only run for each variable separately;
     VAR income;
     OUTPUT OUT=work.Means 
            MEAN(income)=meanincome STDERR(income)=SEincome;
RUN; TITLE;

* Compute -1 and +1 SE away from means;
DATA work.Means; SET work.Means;
     lowerSEincome=meanincome-SEincome;
     upperSEincome=meanincome+SEincome;
RUN;

* AGE MODELS;

TITLE "SAS Line Plot for Income Means Per Year of Age";
PROC SGPLOT DATA=work.Means; WHERE NMISS(age)=0;
     SERIES x=age y=meanincome  / LINEATTRS=(COLOR=BLACK PATTERN=SOLID);
     SERIES x=age y=lowerSEincome / LINEATTRS=(COLOR=RED PATTERN=SOLID);
     SERIES x=age y=upperSEincome / LINEATTRS=(COLOR=RED PATTERN=SOLID);
     XAXIS LABEL="Years of Age" VALUES=(18 TO 76 BY 2);
     YAXIS LABEL="1000s of Income" VALUES=(0 TO 55 BY 5);
RUN;

* Compute means of predicted income from age for each unique value of age;
PROC SORT DATA=work.PredIncomebyAge; BY age18; RUN;
PROC MEANS NOPRINT DATA=work.PredIncomebyAge; BY age18; VAR YhatAge SEyhatAge;
     OUTPUT OUT=work.MeanYhatAge MEAN(YhatAge)=YhatAge MEAN(SEyhatAge)=SEyhatAge; RUN;
* Compute uncentered age for plot;
DATA work.MeanYhatAge; SET work.MeanYhatAge;
     age=age18+18; RUN;
* Merge with means data, compute -1 and +1 SE for yhat from age;
DATA work.Means; MERGE work.Means work.MeanYhatAge; BY age; 
     lowerSEage=YhatAge-SEyhatAge;
     upperSEage=YhatAge+SEyhatAge; RUN;
     
TITLE "SAS Line Plot for Income Means Per Year of Age";
PROC SGPLOT DATA=work.Means; WHERE NMISS(age)=0;
     SERIES x=age y=meanincome  / LINEATTRS=(COLOR=BLACK PATTERN=SOLID);
     SERIES x=age y=lowerSEincome / LINEATTRS=(COLOR=RED PATTERN=SOLID);
     SERIES x=age y=upperSEincome / LINEATTRS=(COLOR=RED PATTERN=SOLID);
     SERIES x=age y=YhatAge  / LINEATTRS=(COLOR=BLACK PATTERN=DASH);
     SERIES x=age y=lowerSEage / LINEATTRS=(COLOR=RED PATTERN=DASH);
     SERIES x=age y=upperSEage / LINEATTRS=(COLOR=RED PATTERN=DASH);
     XAXIS LABEL="Years of Age" VALUES=(18 TO 76 BY 2);
     YAXIS LABEL="1000s of Income" VALUES=(-20 TO 60 BY 10);
RUN;

* EDUC MODELS;

TITLE "SAS Line Plot for Income Means Per Year of Education";
PROC SGPLOT DATA=work.Means; WHERE NMISS(educ)=0;
     SERIES x=educ y=meanincome  / LINEATTRS=(COLOR=BLACK PATTERN=SOLID);
     SERIES x=educ y=lowerSEincome / LINEATTRS=(COLOR=RED PATTERN=SOLID);
     SERIES x=educ y=upperSEincome / LINEATTRS=(COLOR=RED PATTERN=SOLID);
     XAXIS LABEL="Years of Education" VALUES=(2 TO 20 BY 1);
     YAXIS LABEL="1000s of Income" VALUES=(0 TO 40 BY 5);
     REFLINE 11 / AXIS=X;
     REFLINE 12 / AXIS=X;
RUN;

* Compute means of predicted income from educ for each unique value of educ;
PROC SORT DATA=work.PredIncomebyEduc; BY educ; RUN;
PROC MEANS NOPRINT DATA=work.PredIncomebyEduc; BY educ; VAR YhatEduc SEyhatEduc;
     OUTPUT OUT=work.MeanYhatEduc MEAN(YhatEduc)=YhatEduc MEAN(SEyhatEduc)=SEyhatEduc; RUN;
* Merge with means data, compute -1 and +1 SE for yhat from educ;
PROC SORT DATA=work.Means; BY educ; RUN;
DATA work.Means; MERGE work.Means work.MeanYhatEduc; BY educ; 
     lowerSEeduc=YhatEduc-SEyhatEduc;
     upperSEeduc=YhatEduc+SEyhatEduc; RUN;

TITLE "SAS Predicted Income By Year of Education: Linear OverHS";
PROC SGPLOT DATA=work.Means; WHERE NMISS(educ)=0;
     SERIES x=educ y=meanincome  / LINEATTRS=(COLOR=BLACK PATTERN=SOLID);
     SERIES x=educ y=lowerSEincome / LINEATTRS=(COLOR=RED PATTERN=SOLID);
     SERIES x=educ y=upperSEincome / LINEATTRS=(COLOR=RED PATTERN=SOLID);
     SERIES x=educ y=YhatEduc  / LINEATTRS=(COLOR=BLACK PATTERN=DASH);
     SERIES x=educ y=lowerSEeduc / LINEATTRS=(COLOR=RED PATTERN=DASH);
     SERIES x=educ y=upperSEeduc / LINEATTRS=(COLOR=RED PATTERN=DASH);
     XAXIS LABEL="Years of Education" VALUES=(2 TO 20 BY 1);
     YAXIS LABEL="1000s of Income" VALUES=(0 TO 40 BY 5);
     REFLINE 11 / AXIS=X;
     REFLINE 12 / AXIS=X;
     REFLINE 15 / AXIS=X;
     REFLINE 16 / AXIS=X;
     REFLINE 17 / AXIS=X;
     REFLINE 18 / AXIS=X;
RUN;

* HAPPY MODELS;

TITLE "SAS Line Plot for Income Means Per Level of Happy";
PROC SGPLOT DATA=work.Means; WHERE NMISS(happy)=0;
     SERIES x=happy y=meanincome  / LINEATTRS=(COLOR=BLACK PATTERN=SOLID);
     *SERIES x=happy y=lowerSEincome / LINEATTRS=(COLOR=RED PATTERN=SOLID);
     *SERIES x=happy y=upperSEincome / LINEATTRS=(COLOR=RED PATTERN=SOLID);
     XAXIS LABEL="Ordinal Happiness" VALUES=(1 TO 5 BY 1);
     YAXIS LABEL="1000s of Income" VALUES=(0 TO 30 BY 5);
RUN;

* Compute means of predicted income from age for each unique value of age;
PROC SORT DATA=work.PredIncomebyHappy1; BY happy; RUN;
PROC MEANS NOPRINT DATA=work.PredIncomebyHappy1; BY happy; VAR Yhat1Happy SEyhat1Happy;
     OUTPUT OUT=work.MeanYhat1Happy MEAN(Yhat1Happy)=Yhat1Happy MEAN(SEyhat1Happy)=SEyhat1Happy; RUN;
* Merge with means data, compute -1 and +1 SE for yhat from age;
PROC SORT DATA=work.Means; BY happy; RUN;
DATA work.Means; MERGE work.Means work.MeanYhat1Happy; BY happy; 
     lowerSE1Happy=Yhat1Happy-SEyhat1Happy;
     upperSE1Happy=Yhat1Happy+SEyhat1Happy; RUN;
     
TITLE "SAS Line Plot for Income Means Per Level of Happy: Linear Slope";
PROC SGPLOT DATA=work.Means; WHERE NMISS(happy)=0;
     SERIES x=happy y=meanincome  / LINEATTRS=(COLOR=BLACK PATTERN=SOLID);
     SERIES x=happy y=lowerSEincome / LINEATTRS=(COLOR=RED PATTERN=SOLID);
     SERIES x=happy y=upperSEincome / LINEATTRS=(COLOR=RED PATTERN=SOLID);
     SERIES x=happy y=Yhat1Happy  / LINEATTRS=(COLOR=BLACK PATTERN=DASH);
     SERIES x=happy y=lowerSE1Happy / LINEATTRS=(COLOR=RED PATTERN=DASH);
     SERIES x=happy y=upperSE1Happy / LINEATTRS=(COLOR=RED PATTERN=DASH);
     XAXIS LABEL="Ordinal Happiness" VALUES=(1 TO 5 BY 1);
     YAXIS LABEL="1000s of Income" VALUES=(5 TO 25 BY 5);
RUN;


* Close output;
ODS RTF CLOSE;
